﻿using System.Windows;

namespace TestProject
{
    /// <summary>
    /// 윈도우 종료자
    /// </summary>
    public class WindowCloser
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// Dependency Property
        ////////////////////////////////////////////////////////////////////////////////////////// Static
        //////////////////////////////////////////////////////////////////////////////// Public

        #region 윈도우 닫기 이용 가능 여부 첨부 속성 - EnableWindowClosingProperty

        /// <summary>
        /// 윈도우 닫기 이용 가능 여부 속성
        /// </summary>
        public static readonly DependencyProperty EnableWindowClosingProperty = DependencyProperty.RegisterAttached
        (
            "EnableWindowClosing",
            typeof(bool),
            typeof(WindowCloser),
            new PropertyMetadata
            (
                false,
                EnableWindowClosingPropertyChangedCallback
            )
        );

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Method
        ////////////////////////////////////////////////////////////////////////////////////////// Static
        //////////////////////////////////////////////////////////////////////////////// Public

        #region 윈도우 닫기 이용 가능 여부 값 구하기 - GetEnableWindowClosing(d)

        /// <summary>
        /// 윈도우 닫기 이용 가능 여부 값 구하기
        /// </summary>
        /// <param name="d">의존 객체</param>
        /// <returns>윈도우 닫기 이용 가능 여부 값</returns>
        public static bool GetEnableWindowClosing(DependencyObject d)
        {
            return (bool)d.GetValue(EnableWindowClosingProperty);
        }

        #endregion
        #region 윈도우 닫기 이용 가능 여부 값 설정하기 - SetEnableWindowClosing(d, value)

        /// <summary>
        /// 윈도우 닫기 이용 가능 여부 값 설정하기
        /// </summary>
        /// <param name="d">의존 객체</param>
        /// <param name="value">윈도우 닫기 이용 가능 여부 값</param>
        public static void SetEnableWindowClosing(DependencyObject d, bool value)
        {
            d.SetValue(EnableWindowClosingProperty, value);
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////// Private

        #region 윈도우 닫기 이용 가능 여부 속성 변경시 콜백 처리하기 - EnableWindowClosingPropertyChangedCallback(d, e)

        /// <summary>
        /// 윈도우 닫기 이용 가능 여부 속성 변경시 콜백 처리하기
        /// </summary>
        /// <param name="d">의존 객체</param>
        /// <param name="e">이벤트 인자</param>
        private static void EnableWindowClosingPropertyChangedCallback(DependencyObject d, DependencyPropertyChangedEventArgs e)
        {
            if(d is Window window)
            {
                window.Loaded += (s2, e2) =>
                {
                    if(window.DataContext is ICloseWindow vm)
                    {
                        vm.CloseAction += () =>
                        {
                            window.Close();
                        };

                        window.Closing += (s3, e3) =>
                        {
                            e3.Cancel = !vm.CanClose();
                        };
                    }
                };
            }
        }

        #endregion
    }
}